__author__ = 'cliburn'

import matplotlib
matplotlib.use('Agg')
import os, sys, glob
import numpy as np
import numpy.random as npr
import fcm
import fcm.statistics as stats
from fcm.graphics import bilinear_interpolate
# import utils_cc as utils
import pylab
import copy
import utils_cc as utils
import time

class Timer:
    def __enter__(self):
        self.start = time.clock()
        return self

    def __exit__(self, *args):
        self.end = time.clock()
        self.interval = self.end - self.start


def hdp_modal_cluster(xs, nclusts, niter, burnin, tune_interval, verbose=True, device=1, prior=None, e0=None, f0=None, g0=None, h0=None, rs=None):
    """Performs the following sequence of events
    1. Fits an HDP model to the list of dataset in xs
    2. Performs modal clustering on each dataset
    3. Use an averaged posterior as reference
    4. Sets reference cmap as common cmap for aligned modal clusters across data sets
    5. Returns list of fitted comonnent restuls (rs) and list of fitted modal results (cs)
    """

    if rs is None:
        m = stats.HDPMixtureModel(nclusts, niter, burnin=burnin)
        m.device = device
        m.ident = True
        m.parallel = True
        if e0:
            m.e0 = e0
        if f0:
            m.f0 = f0
        if g0:
            m.g0 = g0
        if h0:
            m.h0 = h0
        if prior:
            m.prior_pi = prior.pis
            m.prior_mu = prior.mus
            m.prior_sigma = prior.sigmas
        rss = m.fit(xs, verbose=verbose, tune_interval=tune_interval)

        rs = [r.average() for r in rss]
    else:
        rss = []

    # set reference cmap by averaging
    r_consensus = copy.deepcopy(rs[0])
    pis = np.sum([r.pis for r in rs], 0)
    pis /= np.sum(pis)
    for i in range(len(r_consensus.clusters)):
        r_consensus.clusters[i].pi = pis[i]

    # merge aggressively
    c_consensus = r_consensus.make_modal(delta=0.1)
    ref_cmap = c_consensus.cmap

    cs = []
    for r in rs:
        cs.append(stats.ModalDPMixture(
            r.clusters, ref_cmap, c_consensus.modemap, m=0, s=1))
    cs.append(c_consensus)

    return rss, rs, cs

if __name__ == '__main__':
    results_dir = '../results'
    prior = None

    print "python fit_data.py <npy_file> <niter> <burnin> <nclusts> <seed> <device>"
    target = sys.argv[1]
    niter = int(sys.argv[2])
    burnin = int(sys.argv[3])
    nclusts = int(sys.argv[4])
    seed = int(sys.argv[5])
    device = int(sys.argv[6])
    e0 = float(sys.argv[7])
    f0 = float(sys.argv[8])
    g0 = float(sys.argv[9])
    h0 = float(sys.argv[10])
    tune_interval = 100
    verbose = 25
    path, filename = os.path.split(target)
    name, ext = os.path.splitext(filename)
    xss = np.load(target)

    identifier = '%s_%s_k%d_i%d_b%d_s%d_e%.2f_f%.2f_g%.2f_h%.2f_rep0' % ('plos', name, nclusts, niter, burnin, seed, e0, f0, g0, h0)
    if not os.path.exists(os.path.join(results_dir, '%s_cs.pic.gz' % identifier)):
        with Timer() as t:
            rss, rs, cs = hdp_modal_cluster(xss, nclusts, niter, burnin, tune_interval, verbose, device, prior, e0, f0, g0, h0)
        with open(name + '_plostime.txt', 'a') as fo:
            fo.write('%.03f\n' % t.interval)
        print target, t.interval

        if os.path.exists(os.path.join(results_dir, '%s_cs.pic.gz' % identifier)):
            rep = int(identifier[-1]) + 1
            identifier = identifier[:-1] + str(rep)

        utils.pickle(os.path.join(results_dir, '%s_rss.pic.gz' % identifier), rss)
        utils.pickle(os.path.join(results_dir, '%s_rs.pic.gz' % identifier),  rs)
        utils.pickle(os.path.join(results_dir, '%s_cs.pic.gz' % identifier), cs)
