import glob
import numpy as np
import numpy.random as npr
import fcm
import fcm.statistics as stats
import pylab
import copy

if __name__ == '__main__':

    # HDPGMM settings
    # IMPORTANT: burnin and niter are set to extremely low values for use as a test case on non-CUDA machines
    nclusts = 128 # upper limit for number of components in truncated DP
    niter = 20 # number of iterations to use for posterior distribution (manusript uses 2000)
    burnin = 200 # number of burnin iterations (manuscript uses 20000)

    device = False # use gpu
    # device = [0,1] # use two GPU devices
    verbose = 10 # report every 10 MCMC steps
    seed = 201 # random number seed

    # load all data files into list of numpy arrays
    xs = [np.load(f) for f in glob.glob('data/*npy')]

    # set up model for fitting to data
    m = stats.HDPMixtureModel(nclusts, niter, burnin=burnin)
    m.device = device
    m.ident = True
    m.parallel = True

    # set random number seed
    npr.seed(seed)

    # find posterior distribution averaged over niter MCMC interations
    rs = [r.average() for r in m.fit(xs, verbose=verbose)]

    # use modes from the average posterior distributions over all data samples
    r_consensus = copy.deepcopy(rs[0])
    pis = np.sum([r.pis for r in rs], 0)
    pis /= np.sum(pis)
    for i in range(len(r_consensus.clusters)):
        r_consensus.clusters[i].pi = pis[i]
    c_consensus = r_consensus.make_modal()
    ref_cmap = c_consensus.cmap
    cs = [stats.ModalDPMixture(r.clusters, ref_cmap, c_consensus.modemap, 0, 1) for r in rs]

    # make plots
    markers = ['FSC', 'SSC', 'CD3', 'CD45', 'Tet']
    dims = [(0,1), (2,3), (3,4)]
    nrows = len(xs)
    ncols = len(dims)
    pylab.figure(figsize=(2*ncols, 2*nrows))

    for j, x in enumerate(xs):
        # find assignment of each event to a modal cluster
        z = cs[j].classify(x, use_gpu=False)
        centroids = cs[j].modes()

        for k, dim in enumerate(dims):
            pylab.subplot(nrows, ncols, j*ncols+k+1)
            # color event by modal cluster assignment
            pylab.scatter(x[:, dim[0]], x[:, dim[1]],
                          s=1, c=z, edgecolors='none')
            # show location of modes
            for h, m in enumerate(centroids):
                pylab.text(m[dim[0]], m[dim[1]], str(h),
                           va='center', ha='center',
                           bbox=dict(facecolor='yellow', alpha=0.5))

            pylab.axis([0,1024,0,1024])
            pylab.xticks([])
            pylab.yticks([])
            pylab.xlabel(markers[dim[0]])
            pylab.ylabel(markers[dim[1]])
    pylab.tight_layout()
    pylab.show()
